/* GKrellM-fileread
 
    Authors:
        
    Henry Palonen <h@yty.net> (original version for gkrellm1)
    Thomas Moestl <tmoestl@gmx.net> (support for multiple files)
    Tomas Styblo  <tripie@cpan.org> (version for gkrellm2)
       
  
   Copyright (C) 1999-2003
 
   This program is free software which I release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
   of that license as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   To get a copy of the GNU General Puplic License,  write to the
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gkrellm2/gkrellm.h>
#include <utime.h>
#include <dirent.h>
#include <errno.h>

#define DATA_BUFLEN 1024

typedef struct
{
    gchar    *path;
    GkrellmPanel	 *fileread;
    GkrellmDecal	 *decal_fileread;
} item_t;

static GkrellmMonitor *monitor;
static GArray   *fileread_file_paths;
static gint     fileread_file_path_num;
static gint     ctrl_path_num;
GtkWidget       *fileread_vbox;

void setup_fileread();
void destroy_fileread();

/*
 *	Drawing actual text-string
 */

    static void
draw_fileread(gchar data[DATA_BUFLEN], gint i)
{
    GdkPixmap *bg_scaled = NULL;
    GkrellmDecal *decal_bg = NULL;
    GkrellmPiximage *bg = NULL;
    static GtkTooltips *tooltip = NULL;
    gchar *tooltip_text = NULL;
    
    if ((bg = gkrellm_bg_meter_piximage(DEFAULT_STYLE_ID))) {
        gkrellm_scale_piximage_to_pixmap(bg, &bg_scaled, NULL, 
                gkrellm_chart_width(), gkrellm_panel_configure_get_height(g_array_index(fileread_file_paths,item_t, i).fileread));
    }
    
    if (bg_scaled) {
        decal_bg = gkrellm_create_decal_pixmap(g_array_index(fileread_file_paths,item_t, i).fileread, bg_scaled, NULL, 0, NULL, 0, 0);
        gkrellm_draw_decal_pixmap(g_array_index(fileread_file_paths,item_t, i).fileread, decal_bg, 0);
    }

    gkrellm_remove_decal(g_array_index(fileread_file_paths,item_t, i).fileread, g_array_index(fileread_file_paths,item_t, i).decal_fileread);
    gkrellm_insert_decal(g_array_index(fileread_file_paths,item_t, i).fileread, g_array_index(fileread_file_paths,item_t, i).decal_fileread, TRUE);
    gkrellm_draw_decal_text(g_array_index(fileread_file_paths,item_t, i).fileread, 
            g_array_index(fileread_file_paths,item_t, i).decal_fileread, data,
            -1);

    gkrellm_draw_panel_layers(g_array_index(fileread_file_paths,item_t, i).fileread);
   
    /* tooltip - must be in unicode */
    if (! tooltip)
        tooltip = gtk_tooltips_new();
    
    if (g_utf8_validate(data, -1, NULL)) {
        tooltip_text = g_strdup_printf("%s (%s)", data, g_array_index(fileread_file_paths,item_t, i).path);
    }
    else {
        gchar *data_utf8 = g_locale_to_utf8(data, -1, NULL, NULL, NULL);
        if (data_utf8) {
            tooltip_text = g_strdup_printf("%s (%s)", data_utf8, g_array_index(fileread_file_paths,item_t, i).path);
            g_free(data_utf8);
        }
        else {
            tooltip_text = g_strdup_printf("ERROR: Cannot convert data to UTF-8 (%s)", g_array_index(fileread_file_paths,item_t, i).path);
        }
    }
    gtk_tooltips_set_tip(tooltip, g_array_index(fileread_file_paths,item_t, i).fileread->drawing_area, tooltip_text, NULL);
    g_free(tooltip_text);
}

/*
 *  Updating monitor value
 *
 *  data must point to preallocated char[DATA_BUFLEN] buffer.
 */
    static void
update_fileread_vals(char *path, char *data)
{
    FILE *f;
    int i;
    size_t len;

    if (gkrellm_plugin_debug())
        printf("path: %s\n",path);

    if ((f = fopen(path, "r")) != NULL)
    {
        len = fread(data, sizeof(char), DATA_BUFLEN, f);
        data[len] = '\0';
        fclose(f);

        /* use only the first line without the newline at the end */
        for (i = 0; i < strlen(data); i++) {
            if (data[i] == '\r' || data[i] == '\n') {
                data[i] = '\0';
                break;
            }
        }
    }
    else 
    {
        snprintf(data, DATA_BUFLEN, "I/O error: %s (%d)", strerror(errno), errno);
    }
}

    static void
update_fileread_data()
{
    gchar   temp[DATA_BUFLEN];
    gint    i;

    if (gkrellm_plugin_debug())
        printf("update: %i\n",fileread_file_path_num);
    for (i=0;i<fileread_file_path_num;i++)
    {
        update_fileread_vals(g_array_index(fileread_file_paths,item_t,i).path, temp);

        if (gkrellm_plugin_debug())
            printf("draw: %s (%i)\n",temp,i);

        draw_fileread(temp,i);
    }
}

    static void
update_fileread()
{

    /* It's enough to update once every ten seconds. */

    if (GK.ten_second_tick)
    {
        update_fileread_data();
    }
}	

    static gint
fileread_mouse_button (GtkWidget *widget, GdkEventButton *ev)
{
    if (ev->button == 3)
    {
        gkrellm_open_config_window(monitor);
    }
    return TRUE;
}

    static gint
fileread_expose_event (GtkWidget *widget, GdkEventExpose *event)
{
    gint i;

    for (i=0;i<fileread_file_path_num;i++) 
    {
        if (widget == g_array_index(fileread_file_paths,item_t, i).fileread->drawing_area)
        {
            gdk_draw_pixmap(widget->window,
                    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                    g_array_index(fileread_file_paths,item_t, i).fileread->pixmap,
                    event->area.x, event->area.y,
                    event->area.x, event->area.y,
                    event->area.width, event->area.height);
        }

    }
    return FALSE;
}

/*
 *	Creating fileread-monitor
 */

    void 
destroy_fileread()
{
    gint i;

    for (i=0;i<fileread_file_path_num;i++) 
    {
        /* GK.monitor_height -= g_array_index(fileread_file_paths,item_t, i).fileread->h; */
        gkrellm_destroy_decal_list(g_array_index(fileread_file_paths,item_t, i).fileread);
        gkrellm_panel_destroy(g_array_index(fileread_file_paths,item_t, i).fileread);
        g_array_index(fileread_file_paths,item_t, i).fileread=NULL;
    }

}

    void 
setup_fileread()
{ 
    GkrellmStyle	*fileread_style;
    GkrellmTextstyle               *ts;
    gint                    w,i,first_create;

#ifdef USE_FS_STYLE_WHICH_MAY_SHOW_A_DIFF_ONLY_IN_THE_DEFAULT_THEME
#define MY_STYLE        FS_STYLE
#else
#define MY_STYLE        DEFAULT_STYLE_ID
#endif  

    /* Allocate a panel structure.  plug->label and plug->textstyle
       |  structures will also be allocated...
       */

    for (i=0;i<fileread_file_path_num;i++)  
    {
        first_create=!g_array_index(fileread_file_paths,item_t, i).fileread;
        if (first_create)
        {
            g_array_index(fileread_file_paths,item_t, i).fileread = gkrellm_panel_new0();
        }
        else
        {
            gkrellm_destroy_decal_list(g_array_index(fileread_file_paths,item_t, i).fileread);
        }

        fileread_style = gkrellm_meter_style(DEFAULT_STYLE_ID);

        ts = gkrellm_meter_textstyle(DEFAULT_STYLE_ID);
        w = gdk_string_width(ts->font, "Outside: -23.45") + 2;
        if (w > gkrellm_chart_width() - 2 * fileread_style->margin.left)
            w = gkrellm_chart_width() - 2 * fileread_style->margin.left;

        g_array_index(fileread_file_paths,item_t, i).decal_fileread = 
            gkrellm_create_decal_text(g_array_index(fileread_file_paths,item_t, i).fileread, "Out: -53.24", ts, fileread_style, -1, -1, -1);

        gkrellm_panel_configure(g_array_index(fileread_file_paths,item_t, i).fileread,
         NULL, fileread_style);

        gkrellm_panel_create(fileread_vbox, monitor, g_array_index(fileread_file_paths, item_t, i).fileread);

        /* GK.monitor_height += g_array_index(fileread_file_paths,item_t, i).fileread->h; */

        if (first_create)
        {
            gtk_signal_connect(GTK_OBJECT (g_array_index(fileread_file_paths,item_t, i).fileread->drawing_area), 
                    "button_press_event", (GtkSignalFunc) fileread_mouse_button, NULL);

            gtk_signal_connect(GTK_OBJECT (g_array_index(fileread_file_paths,item_t, i).fileread->drawing_area), 
                    "expose_event", (GtkSignalFunc) fileread_expose_event, NULL);
        }
    }
}

    void
create_fileread(GtkWidget *vbox, gint first_create)
{
    fileread_vbox=vbox;
    setup_fileread();
    update_fileread_data();
}

/*
 *	Configuration page
 *	Configuration contains path to file ;)
 */

static GtkWidget                *fileread_path_entry;
static GtkWidget                *fileread_item_list;
static gint                     fileread_item=0;
static gchar			*fileread_info_text =
"The Fileread plugin reads first line from\n"
"a file and displays it in the gkrellm panel.\n" 
"\n"
"Authors:\n"
"\n"
"Henry Palonen <h@yty.net>\n" 
"(original version for gkrellm1)\n"
"\n"
"Thomas Moestl <tmoestl@gmx.net>\n"
"(support for multiple files)\n"
"\n"
"Tomas Styblo <tripie@cpan.org>\n"
"(version for gkrellm2)\n"
"\n"
"Version for gkrellm1:\n"
"http://www.yty.net/h/gkrellm/\n"
"\n"
"Version for gkrellm2:\n"
"http://sweb.cz/tripie/gkrellm/fileread/\n"
"\n"
"Copyright (C) 1999-2003\n"
"Distributed under the GNU General Public License.\n"
;

/* Shamelessly ripped from the Gtk docs. */
void fr_message(gchar *message) 
{
    GtkWidget *dialog, *label, *okay_button;
    dialog = gtk_dialog_new();
    label = gtk_label_new (message);
    okay_button = gtk_button_new_with_label("OK");
    gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
            GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT(dialog));
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
            okay_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
            label);
    gtk_widget_show_all (dialog);
}


    static void
item_sel(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data )
{
    fileread_item=row;
}

    static void
destroy_item(gpointer data)
{
    g_free(((item_t *)data)->path);
    g_free(data);     
}

    static void
on_add_click(GtkButton *button, gpointer *data)
{
    gchar *txt=g_strdup(gtk_entry_get_text(GTK_ENTRY(fileread_path_entry)));
    gchar *list[1];
    item_t *item;

    if (!txt[0]) 
    {
        fr_message("Please enter a pathname!");
        g_free(txt);
        return;
    } 
    else
    {
        item=g_new(item_t,1);
        item->path=txt;
        list[0]=txt;
        gtk_clist_set_row_data_full(GTK_CLIST(fileread_item_list),
                gtk_clist_append(GTK_CLIST(fileread_item_list),list),
                (gpointer)item, destroy_item);
        gtk_clist_columns_autosize(GTK_CLIST(fileread_item_list));
        ctrl_path_num++;
    }
}

    static void
on_del_click(GtkButton *button, gpointer *data)
{
    if (gtk_clist_get_row_data(GTK_CLIST(fileread_item_list),fileread_item)) 
    {
        gtk_clist_remove(GTK_CLIST(fileread_item_list),fileread_item);
        ctrl_path_num--;
    }
}

    static void
create_fileread_tab(GtkWidget *tab_vbox)
{
    GtkWidget		*vbox;
    GtkWidget		*tabs;
    GtkWidget               *text;
    GtkWidget               *scrolled;
    GtkWidget               *btn_add;
    GtkWidget               *btn_del;
    GtkWidget               *btn_hbox;
    GtkWidget               *data_hbox;
    GtkWidget               *path_lbl;
    gchar                   *titles[1]={"File"};
    gchar                   *tmp[1];
    gint                    i;
    item_t                  *item;
    GtkTextBuffer           *textbuf;

    tabs = gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
    gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

    /* File location tab */

    vbox = gkrellm_gtk_notebook_page(tabs, "File locations");
    data_hbox = gtk_hbox_new(FALSE, 5);
    btn_hbox = gtk_hbox_new(FALSE, 5);
    fileread_path_entry = gtk_entry_new_with_max_length(255);
    btn_add=gtk_button_new_with_label("Add");
    gtk_signal_connect(GTK_OBJECT(btn_add), "clicked",
            GTK_SIGNAL_FUNC(on_add_click),NULL);
    btn_del=gtk_button_new_with_label("Delete");
    gtk_signal_connect(GTK_OBJECT(btn_del), "clicked",
            GTK_SIGNAL_FUNC(on_del_click),NULL);

    path_lbl=gtk_label_new("File:");

    fileread_item_list=gtk_clist_new_with_titles(1,titles);
    gtk_signal_connect(GTK_OBJECT(fileread_item_list), "select-row",
            GTK_SIGNAL_FUNC(item_sel),
            NULL);
    gtk_clist_set_selection_mode(GTK_CLIST(fileread_item_list),GTK_SELECTION_SINGLE);

    ctrl_path_num=fileread_file_path_num;
    for (i=0;i<ctrl_path_num;i++)
    {
        tmp[0]=g_array_index(fileread_file_paths,item_t,i).path;
        item=g_new(item_t,1);
        item->path=g_strdup(g_array_index(fileread_file_paths,item_t,i).path);
        gtk_clist_set_row_data_full(GTK_CLIST(fileread_item_list),
                gtk_clist_append(GTK_CLIST(fileread_item_list),tmp),
                (gpointer)item, destroy_item);
    }
    fileread_item=0;
    if (ctrl_path_num==0)
        fileread_item=-1;
    gtk_clist_columns_autosize(GTK_CLIST(fileread_item_list));

    gtk_box_pack_start(GTK_BOX(data_hbox), path_lbl, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(data_hbox), fileread_path_entry, TRUE, TRUE, 2);

    gtk_box_pack_start(GTK_BOX(btn_hbox), btn_add, TRUE, TRUE, 2);
    gtk_box_pack_start(GTK_BOX(btn_hbox), btn_del, TRUE, TRUE, 2);

    gtk_box_pack_start(GTK_BOX(vbox), data_hbox, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(vbox), btn_hbox, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(vbox), fileread_item_list, TRUE, TRUE, 2);

    /* Info tab */

    vbox = gkrellm_gtk_notebook_page(tabs, "Info");
    scrolled = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
    textbuf = gtk_text_buffer_new(NULL);
    gtk_text_buffer_set_text(textbuf, fileread_info_text, -1);
    text = gtk_text_view_new_with_buffer(textbuf);
    gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
    gtk_container_add(GTK_CONTAINER(scrolled), text);

}

/*
 *	Saving config
 */

    static void
save_fileread_config(FILE *f)
{
    gint i;
    fprintf(f,"fileread num %i\n",fileread_file_path_num);
    for (i=0;i<fileread_file_path_num;i++)
    {
        fprintf(f,"fileread filename%i %s\n",i,g_array_index(fileread_file_paths,item_t,i).path);
    }
}

/*
 *  Loading config
 */

static item_t  c;

    static void
load_fileread_config(gchar *arg)
{
    gchar   item[256];
    gint    n,i;

    /* printf("fileread_config=<%s>\n", arg); */

    n = sscanf(arg, "num %i",&i);
    if (n == 1) 
    {
        fileread_file_path_num=i; 
    } 
    else 
    {
        n = sscanf(arg, "filename%i %s", &i,item);
        if (n == 2) 
        {
            c.path=g_strdup(item);
            c.fileread=NULL;
            c.decal_fileread=NULL;
            g_array_append_val(fileread_file_paths,c);
        }
    } 
}

/*
 *	Applying config.
 *	This includes only updating fileread_file_path variable.
 */

    static void
apply_fileread_config()
{
    gint                   i;
    item_t                 item; 

    destroy_fileread();

    for (i=0;i<fileread_file_path_num;i++) 
    {
        g_free(g_array_index(fileread_file_paths,item_t, i).path);
    }
    fileread_file_path_num=ctrl_path_num;
    g_array_free(fileread_file_paths,TRUE);
    fileread_file_paths=g_array_new(FALSE,TRUE,sizeof(item_t));
    for (i=0;i<ctrl_path_num;i++) 
    {
        item.path=g_strdup(((item_t *)gtk_clist_get_row_data(GTK_CLIST(fileread_item_list),i))->path);
        item.fileread=NULL;
        item.decal_fileread=NULL;
        g_array_append_val(fileread_file_paths,item);
    }
    setup_fileread();
    update_fileread_data();
}


static GkrellmMonitor plugin_mon =
{
    "Fileread",               	    /* Name, for config tab.    */
    0,                                  /* Id,  0 if a plugin       */
    create_fileread,          	    /* The create function      */
    update_fileread,          	    /* The update function      */
    create_fileread_tab,                /* The config tab create function   */
    apply_fileread_config,              /* Apply the config function        */
    save_fileread_config,               /* Save user config */
    load_fileread_config,               /* Load user config */
    "fileread",                         /* config keyword */
    NULL,                               /* Undefined 2  */
    NULL,                               /* Undefined 1  */
    NULL,                               /* Undefined 0  */
    MON_APM,                            /* Insert plugin before this monitor */
    NULL,                               /* Handle if a plugin, filled in by GKrellM     */
    NULL                                /* path if a plugin, filled in by GKrellM       */
};

    GkrellmMonitor *
gkrellm_init_plugin()
{
    fileread_file_paths=g_array_new(FALSE,TRUE,sizeof(item_t));
    fileread_file_path_num=0;
    monitor = &plugin_mon;
    return &plugin_mon;
}
